@@ -11,12 +11,17 @@ from sqlalchemy import cast
 from sqlalchemy import JSON
 from sqlalchemy import schema
 from sqlalchemy import sql
+from sqlalchemy.ext.compiler import compiles
 
+from .base import alter_table
+from .base import format_table_name
+from .base import RenameTable
 from .impl import DefaultImpl
 from .. import util
 
 if TYPE_CHECKING:
     from sqlalchemy.engine.reflection import Inspector
+    from sqlalchemy.sql.compiler import DDLCompiler
     from sqlalchemy.sql.elements import Cast
     from sqlalchemy.sql.elements import ClauseElement
     from sqlalchemy.sql.schema import Column
@@ -36,7 +41,7 @@ class SQLiteImpl(DefaultImpl):
     """
 
     def requires_recreate_in_batch(
-        self, batch_op: "BatchOperationsImpl"
+        self, batch_op: BatchOperationsImpl
     ) -> bool:
         """Return True if the given :class:`.BatchOperationsImpl`
         would need the table to be recreated and copied in order to
@@ -63,7 +68,7 @@ class SQLiteImpl(DefaultImpl):
         else:
             return False
 
-    def add_constraint(self, const: "Constraint"):
+    def add_constraint(self, const: Constraint):
         # attempt to distinguish between an
         # auto-gen constraint and an explicit one
         if const._create_rule is None:  # type:ignore[attr-defined]
@@ -80,7 +85,7 @@ class SQLiteImpl(DefaultImpl):
                 "SQLite migrations using a copy-and-move strategy."
             )
 
-    def drop_constraint(self, const: "Constraint"):
+    def drop_constraint(self, const: Constraint):
         if const._create_rule is None:  # type:ignore[attr-defined]
             raise NotImplementedError(
                 "No support for ALTER of constraints in SQLite dialect. "
@@ -90,8 +95,8 @@ class SQLiteImpl(DefaultImpl):
 
     def compare_server_default(
         self,
-        inspector_column: "Column",
-        metadata_column: "Column",
+        inspector_column: Column,
+        metadata_column: Column,
         rendered_metadata_default: Optional[str],
         rendered_inspector_default: Optional[str],
     ) -> bool:
@@ -135,8 +140,8 @@ class SQLiteImpl(DefaultImpl):
 
     def autogen_column_reflect(
         self,
-        inspector: "Inspector",
-        table: "Table",
+        inspector: Inspector,
+        table: Table,
         column_info: Dict[str, Any],
     ) -> None:
         # SQLite expression defaults require parenthesis when sent
@@ -147,11 +152,11 @@ class SQLiteImpl(DefaultImpl):
             column_info["default"] = "(%s)" % (column_info["default"],)
 
     def render_ddl_sql_expr(
-        self, expr: "ClauseElement", is_server_default: bool = False, **kw
+        self, expr: ClauseElement, is_server_default: bool = False, **kw
     ) -> str:
         # SQLite expression defaults require parenthesis when sent
         # as DDL
-        str_expr = super(SQLiteImpl, self).render_ddl_sql_expr(
+        str_expr = super().render_ddl_sql_expr(
             expr, is_server_default=is_server_default, **kw
         )
 
@@ -164,9 +169,9 @@ class SQLiteImpl(DefaultImpl):
 
     def cast_for_batch_migrate(
         self,
-        existing: "Column",
-        existing_transfer: Dict[str, Union["TypeEngine", "Cast"]],
-        new_type: "TypeEngine",
+        existing: Column,
+        existing_transfer: Dict[str, Union[TypeEngine, Cast]],
+        new_type: TypeEngine,
     ) -> None:
         if (
             existing.type._type_affinity  # type:ignore[attr-defined]
@@ -178,6 +183,16 @@ class SQLiteImpl(DefaultImpl):
             )
 
 
+@compiles(RenameTable, "sqlite")
+def visit_rename_table(
+    element: RenameTable, compiler: DDLCompiler, **kw
+) -> str:
+    return "%s RENAME TO %s" % (
+        alter_table(compiler, element.table_name, element.schema),
+        format_table_name(compiler, element.new_table_name, None),
+    )
+
+
 # @compiles(AddColumn, 'sqlite')
 # def visit_add_column(element, compiler, **kw):
 #    return "%s %s" % (
