@@ -1,5 +1,3 @@
-import sys
-
 from sqlalchemy import BIGINT
 from sqlalchemy import BigInteger
 from sqlalchemy import Boolean
@@ -17,6 +15,7 @@ from sqlalchemy import Index
 from sqlalchemy import inspect
 from sqlalchemy import INTEGER
 from sqlalchemy import Integer
+from sqlalchemy import JSON
 from sqlalchemy import LargeBinary
 from sqlalchemy import MetaData
 from sqlalchemy import Numeric
@@ -33,7 +32,6 @@ from sqlalchemy import UniqueConstraint
 from sqlalchemy import VARCHAR
 from sqlalchemy.dialects import mysql
 from sqlalchemy.dialects import sqlite
-from sqlalchemy.testing import in_
 from sqlalchemy.types import NULLTYPE
 from sqlalchemy.types import VARBINARY
 
@@ -48,16 +46,18 @@ from alembic.testing import eq_
 from alembic.testing import is_
 from alembic.testing import is_not_
 from alembic.testing import mock
+from alembic.testing import schemacompare
 from alembic.testing import TestBase
 from alembic.testing.env import clear_staging_env
 from alembic.testing.env import staging_env
+from alembic.testing.suite._autogen_fixtures import _default_name_filters
+from alembic.testing.suite._autogen_fixtures import _default_object_filters
+from alembic.testing.suite._autogen_fixtures import AutogenFixtureTest
+from alembic.testing.suite._autogen_fixtures import AutogenTest
 from alembic.util import CommandError
-from ._autogen_fixtures import _default_name_filters
-from ._autogen_fixtures import _default_object_filters
-from ._autogen_fixtures import AutogenFixtureTest
-from ._autogen_fixtures import AutogenTest
 
-py3k = sys.version_info >= (3,)
+# TODO: we should make an adaptation of CompareMetadataToInspectorTest that is
+#       more well suited towards generic backends (2021-06-10)
 
 
 class AutogenCrossSchemaTest(AutogenTest, TestBase):
@@ -289,7 +289,7 @@ class AutogenDefaultSchemaIsNoneTest(AutogenFixtureTest, TestBase):
     __only_on__ = "sqlite"
 
     def setUp(self):
-        super(AutogenDefaultSchemaIsNoneTest, self).setUp()
+        super().setUp()
 
         # in SQLAlchemy 1.4, SQLite dialect is setting this name
         # to "main" as is the actual default schema name for SQLite.
@@ -318,7 +318,7 @@ class AutogenDefaultSchemaIsNoneTest(AutogenFixtureTest, TestBase):
         eq_(len(diffs), 0)
 
 
-class ModelOne(object):
+class ModelOne:
     __requires__ = ("unique_constraint_reflection",)
 
     schema = None
@@ -429,7 +429,10 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         autogenerate._produce_net_changes(ctx, uo)
 
         diffs = uo.as_diffs()
-        eq_(diffs[0], ("add_table", metadata.tables["item"]))
+        eq_(
+            diffs[0],
+            ("add_table", schemacompare.CompareTable(metadata.tables["item"])),
+        )
 
         eq_(diffs[1][0], "remove_table")
         eq_(diffs[1][1].name, "extra")
@@ -458,15 +461,15 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
             diffs[6], "add_fk", "order", ["user_id"], "user", ["id"]
         )
 
-        eq_(diffs[7][0][0], "modify_default")
-        eq_(diffs[7][0][1], None)
-        eq_(diffs[7][0][2], "user")
-        eq_(diffs[7][0][3], "a1")
-        eq_(diffs[7][0][6].arg, "x")
+        eq_(diffs[7][0][0], "modify_nullable")
+        eq_(diffs[7][0][5], True)
+        eq_(diffs[7][0][6], False)
 
-        eq_(diffs[8][0][0], "modify_nullable")
-        eq_(diffs[8][0][5], True)
-        eq_(diffs[8][0][6], False)
+        eq_(diffs[8][0][0], "modify_default")
+        eq_(diffs[8][0][1], None)
+        eq_(diffs[8][0][2], "user")
+        eq_(diffs[8][0][3], "a1")
+        eq_(diffs[8][0][6].arg, "x")
 
         eq_(diffs[9][0], "remove_index")
         eq_(diffs[9][1].name, "pw_idx")
@@ -509,13 +512,11 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         )
 
         alter_cols = (
-            set(
-                [
-                    d[2]
-                    for d in self._flatten_diffs(diffs)
-                    if d[0].startswith("modify")
-                ]
-            )
+            {
+                d[2]
+                for d in self._flatten_diffs(diffs)
+                if d[0].startswith("modify")
+            }
             .union(
                 d[3].name
                 for d in self._flatten_diffs(diffs)
@@ -527,7 +528,7 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
                 if d[0] == "add_table"
             )
         )
-        eq_(alter_cols, set(["user_id", "order", "user"]))
+        eq_(alter_cols, {"user_id", "order", "user"})
 
     def test_include_name(self):
         all_names = set()
@@ -579,13 +580,11 @@ class AutogenerateDiffTest(ModelOne, AutogenTest, TestBase):
         )
 
         alter_cols = (
-            set(
-                [
-                    d[2]
-                    for d in self._flatten_diffs(diffs)
-                    if d[0].startswith("modify")
-                ]
-            )
+            {
+                d[2]
+                for d in self._flatten_diffs(diffs)
+                if d[0].startswith("modify")
+            }
             .union(
                 d[3].name
                 for d in self._flatten_diffs(diffs)
@@ -720,7 +719,15 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
 
         diffs = uo.as_diffs()
 
-        eq_(diffs[0], ("add_table", metadata.tables["%s.item" % self.schema]))
+        eq_(
+            diffs[0],
+            (
+                "add_table",
+                schemacompare.CompareTable(
+                    metadata.tables["%s.item" % self.schema]
+                ),
+            ),
+        )
 
         eq_(diffs[1][0], "remove_table")
         eq_(diffs[1][1].name, "extra")
@@ -728,7 +735,12 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
         eq_(diffs[2][0], "add_column")
         eq_(diffs[2][1], self.schema)
         eq_(diffs[2][2], "address")
-        eq_(diffs[2][3], metadata.tables["%s.address" % self.schema].c.street)
+        eq_(
+            schemacompare.CompareColumn(
+                metadata.tables["%s.address" % self.schema].c.street
+            ),
+            diffs[2][3],
+        )
 
         eq_(diffs[3][0], "add_constraint")
         eq_(diffs[3][1].name, "uq_email")
@@ -736,7 +748,12 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
         eq_(diffs[4][0], "add_column")
         eq_(diffs[4][1], self.schema)
         eq_(diffs[4][2], "order")
-        eq_(diffs[4][3], metadata.tables["%s.order" % self.schema].c.user_id)
+        eq_(
+            schemacompare.CompareColumn(
+                metadata.tables["%s.order" % self.schema].c.user_id
+            ),
+            diffs[4][3],
+        )
 
         eq_(diffs[5][0][0], "modify_type")
         eq_(diffs[5][0][1], self.schema)
@@ -755,15 +772,15 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
             source_schema=config.test_schema,
         )
 
-        eq_(diffs[7][0][0], "modify_default")
-        eq_(diffs[7][0][1], self.schema)
-        eq_(diffs[7][0][2], "user")
-        eq_(diffs[7][0][3], "a1")
-        eq_(diffs[7][0][6].arg, "x")
+        eq_(diffs[7][0][0], "modify_nullable")
+        eq_(diffs[7][0][5], True)
+        eq_(diffs[7][0][6], False)
 
-        eq_(diffs[8][0][0], "modify_nullable")
-        eq_(diffs[8][0][5], True)
-        eq_(diffs[8][0][6], False)
+        eq_(diffs[8][0][0], "modify_default")
+        eq_(diffs[8][0][1], self.schema)
+        eq_(diffs[8][0][2], "user")
+        eq_(diffs[8][0][3], "a1")
+        eq_(diffs[8][0][6].arg, "x")
 
         eq_(diffs[9][0], "remove_index")
         eq_(diffs[9][1].name, "pw_idx")
@@ -804,6 +821,7 @@ class CompareTypeSpecificityTest(TestBase):
         (VARCHAR(30), String(30), False),
         (VARCHAR(30), String(40), True),
         (VARCHAR(30), Integer(), True),
+        (VARCHAR(30), String(), False),
         (Text(), String(255), True),
         # insp + metadata types same number of
         # args but are different; they're different
@@ -907,6 +925,8 @@ class CompareMetadataToInspectorTest(TestBase):
         (String(32),),
         (LargeBinary(),),
         (Unicode(32),),
+        (JSON(), config.requirements.json_type),
+        (mysql.LONGTEXT(), config.requirements.mysql),
         (Enum("one", "two", "three", name="the_enum"),),
     )
     def test_introspected_columns_match_metadata_columns(self, cola):
@@ -945,7 +965,7 @@ class CompareMetadataToInspectorTest(TestBase):
             mysql.VARCHAR(200, charset="latin1"),
             mysql.VARCHAR(200, charset="utf-8"),
             True,
-            config.requirements.mysql + config.requirements.sqlalchemy_13,
+            config.requirements.mysql,
         ),
         (
             String(255, collation="utf8_bin"),
@@ -957,7 +977,7 @@ class CompareMetadataToInspectorTest(TestBase):
             String(255, collation="utf8_bin"),
             String(255, collation="latin1_bin"),
             True,
-            config.requirements.mysql + config.requirements.sqlalchemy_13,
+            config.requirements.mysql,
         ),
     )
     def test_string_comparisons(self, cola, colb, expect_changes):
@@ -1224,7 +1244,7 @@ class IncludeFiltersAPITest(AutogenTest, TestBase):
             object_filters=include_object, include_schemas=True
         )
 
-        class ExtFunction(object):
+        class ExtFunction:
             pass
 
         extfunc = ExtFunction()
@@ -1272,81 +1292,6 @@ class PKConstraintUpgradesIgnoresNullableTest(AutogenTest, TestBase):
         eq_(diffs, [])
 
 
-class AlterColumnTest(AutogenFixtureTest, TestBase):
-    __backend__ = True
-
-    @testing.combinations((True,), (False,))
-    @config.requirements.comments
-    def test_all_existings_filled(self, pk):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table("a", m1, Column("x", Integer, primary_key=pk))
-        Table("a", m2, Column("x", Integer, comment="x", primary_key=pk))
-
-        alter_col = self._assert_alter_col(m1, m2, pk)
-        eq_(alter_col.modify_comment, "x")
-
-    @testing.combinations((True,), (False,))
-    @config.requirements.comments
-    def test_all_existings_filled_in_notnull(self, pk):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table("a", m1, Column("x", Integer, nullable=False, primary_key=pk))
-        Table(
-            "a",
-            m2,
-            Column("x", Integer, nullable=False, comment="x", primary_key=pk),
-        )
-
-        self._assert_alter_col(m1, m2, pk, nullable=False)
-
-    @testing.combinations((True,), (False,))
-    @config.requirements.comments
-    def test_all_existings_filled_in_comment(self, pk):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table("a", m1, Column("x", Integer, comment="old", primary_key=pk))
-        Table("a", m2, Column("x", Integer, comment="new", primary_key=pk))
-
-        alter_col = self._assert_alter_col(m1, m2, pk)
-        eq_(alter_col.existing_comment, "old")
-
-    @testing.combinations((True,), (False,))
-    @config.requirements.comments
-    def test_all_existings_filled_in_server_default(self, pk):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a", m1, Column("x", Integer, server_default="5", primary_key=pk)
-        )
-        Table(
-            "a",
-            m2,
-            Column(
-                "x", Integer, server_default="5", comment="new", primary_key=pk
-            ),
-        )
-
-        alter_col = self._assert_alter_col(m1, m2, pk)
-        in_("5", alter_col.existing_server_default.arg.text)
-
-    def _assert_alter_col(self, m1, m2, pk, nullable=None):
-        ops = self._fixture(m1, m2, return_ops=True)
-        modify_table = ops.ops[-1]
-        alter_col = modify_table.ops[0]
-
-        if nullable is None:
-            eq_(alter_col.existing_nullable, not pk)
-        else:
-            eq_(alter_col.existing_nullable, nullable)
-        assert alter_col.existing_type._compare_type_affinity(Integer())
-        return alter_col
-
-
 class AutogenKeyTest(AutogenTest, TestBase):
     __only_on__ = "sqlite"
 
@@ -1504,7 +1449,10 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
 
         diffs = autogenerate.compare_metadata(self.context, metadata)
 
-        eq_(diffs[0], ("add_table", metadata.tables["item"]))
+        eq_(
+            diffs[0],
+            ("add_table", schemacompare.CompareTable(metadata.tables["item"])),
+        )
 
         eq_(diffs[1][0], "remove_table")
         eq_(diffs[1][1].name, "extra")
@@ -1533,15 +1481,15 @@ class CompareMetadataTest(ModelOne, AutogenTest, TestBase):
             diffs[6], "add_fk", "order", ["user_id"], "user", ["id"]
         )
 
-        eq_(diffs[7][0][0], "modify_default")
-        eq_(diffs[7][0][1], None)
-        eq_(diffs[7][0][2], "user")
-        eq_(diffs[7][0][3], "a1")
-        eq_(diffs[7][0][6].arg, "x")
+        eq_(diffs[7][0][0], "modify_nullable")
+        eq_(diffs[7][0][5], True)
+        eq_(diffs[7][0][6], False)
 
-        eq_(diffs[8][0][0], "modify_nullable")
-        eq_(diffs[8][0][5], True)
-        eq_(diffs[8][0][6], False)
+        eq_(diffs[8][0][0], "modify_default")
+        eq_(diffs[8][0][1], None)
+        eq_(diffs[8][0][2], "user")
+        eq_(diffs[8][0][3], "a1")
+        eq_(diffs[8][0][6].arg, "x")
 
         eq_(diffs[9][0], "remove_index")
         eq_(diffs[9][1].name, "pw_idx")
@@ -1666,7 +1614,15 @@ class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
 
         diffs = autogenerate.compare_metadata(context, metadata)
 
-        eq_(diffs[0], ("add_table", metadata.tables["test_schema.item"]))
+        eq_(
+            diffs[0],
+            (
+                "add_table",
+                schemacompare.CompareTable(
+                    metadata.tables["test_schema.item"]
+                ),
+            ),
+        )
 
         eq_(diffs[1][0], "remove_table")
         eq_(diffs[1][1].name, "extra")
@@ -1674,7 +1630,12 @@ class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
         eq_(diffs[2][0], "add_column")
         eq_(diffs[2][1], "test_schema")
         eq_(diffs[2][2], "address")
-        eq_(diffs[2][3], metadata.tables["test_schema.address"].c.street)
+        eq_(
+            schemacompare.CompareColumn(
+                metadata.tables["test_schema.address"].c.street
+            ),
+            diffs[2][3],
+        )
 
         eq_(diffs[3][0], "add_constraint")
         eq_(diffs[3][1].name, "uq_email")
@@ -1682,7 +1643,12 @@ class PGCompareMetaData(ModelOne, AutogenTest, TestBase):
         eq_(diffs[4][0], "add_column")
         eq_(diffs[4][1], "test_schema")
         eq_(diffs[4][2], "order")
-        eq_(diffs[4][3], metadata.tables["test_schema.order"].c.user_id)
+        eq_(
+            schemacompare.CompareColumn(
+                metadata.tables["test_schema.order"].c.user_id
+            ),
+            diffs[4][3],
+        )
 
         eq_(diffs[5][0][0], "modify_nullable")
         eq_(diffs[5][0][5], False)
@@ -1711,42 +1677,54 @@ class OrigObjectTest(TestBase):
     def test_drop_fk(self):
         fk = self.fk
         op = ops.DropConstraintOp.from_constraint(fk)
-        is_(op.to_constraint(), fk)
-        is_(op.reverse().to_constraint(), fk)
+        eq_(op.to_constraint(), schemacompare.CompareForeignKey(fk))
+        eq_(op.reverse().to_constraint(), schemacompare.CompareForeignKey(fk))
 
     def test_add_fk(self):
         fk = self.fk
         op = ops.AddConstraintOp.from_constraint(fk)
-        is_(op.to_constraint(), fk)
-        is_(op.reverse().to_constraint(), fk)
+        eq_(op.to_constraint(), schemacompare.CompareForeignKey(fk))
+        eq_(op.reverse().to_constraint(), schemacompare.CompareForeignKey(fk))
         is_not_(None, op.to_constraint().table)
 
     def test_add_check(self):
         ck = self.ck
         op = ops.AddConstraintOp.from_constraint(ck)
-        is_(op.to_constraint(), ck)
-        is_(op.reverse().to_constraint(), ck)
+        eq_(op.to_constraint(), schemacompare.CompareCheckConstraint(ck))
+        eq_(
+            op.reverse().to_constraint(),
+            schemacompare.CompareCheckConstraint(ck),
+        )
         is_not_(None, op.to_constraint().table)
 
     def test_drop_check(self):
         ck = self.ck
         op = ops.DropConstraintOp.from_constraint(ck)
-        is_(op.to_constraint(), ck)
-        is_(op.reverse().to_constraint(), ck)
+        eq_(op.to_constraint(), schemacompare.CompareCheckConstraint(ck))
+        eq_(
+            op.reverse().to_constraint(),
+            schemacompare.CompareCheckConstraint(ck),
+        )
         is_not_(None, op.to_constraint().table)
 
     def test_add_unique(self):
         uq = self.uq
         op = ops.AddConstraintOp.from_constraint(uq)
-        is_(op.to_constraint(), uq)
-        is_(op.reverse().to_constraint(), uq)
+        eq_(op.to_constraint(), schemacompare.CompareUniqueConstraint(uq))
+        eq_(
+            op.reverse().to_constraint(),
+            schemacompare.CompareUniqueConstraint(uq),
+        )
         is_not_(None, op.to_constraint().table)
 
     def test_drop_unique(self):
         uq = self.uq
         op = ops.DropConstraintOp.from_constraint(uq)
-        is_(op.to_constraint(), uq)
-        is_(op.reverse().to_constraint(), uq)
+        eq_(op.to_constraint(), schemacompare.CompareUniqueConstraint(uq))
+        eq_(
+            op.reverse().to_constraint(),
+            schemacompare.CompareUniqueConstraint(uq),
+        )
         is_not_(None, op.to_constraint().table)
 
     def test_add_pk_no_orig(self):
@@ -1758,15 +1736,15 @@ class OrigObjectTest(TestBase):
     def test_add_pk(self):
         pk = self.pk
         op = ops.AddConstraintOp.from_constraint(pk)
-        is_(op.to_constraint(), pk)
-        is_(op.reverse().to_constraint(), pk)
+        eq_(op.to_constraint(), schemacompare.ComparePrimaryKey(pk))
+        eq_(op.reverse().to_constraint(), schemacompare.ComparePrimaryKey(pk))
         is_not_(None, op.to_constraint().table)
 
     def test_drop_pk(self):
         pk = self.pk
         op = ops.DropConstraintOp.from_constraint(pk)
-        is_(op.to_constraint(), pk)
-        is_(op.reverse().to_constraint(), pk)
+        eq_(op.to_constraint(), schemacompare.ComparePrimaryKey(pk))
+        eq_(op.reverse().to_constraint(), schemacompare.ComparePrimaryKey(pk))
         is_not_(None, op.to_constraint().table)
 
     def test_drop_column(self):
@@ -1789,27 +1767,25 @@ class OrigObjectTest(TestBase):
         t = self.table
 
         op = ops.DropTableOp.from_table(t)
-        is_(op.to_table(), t)
-        is_(op.reverse().to_table(), t)
-        is_(self.metadata, op.to_table().metadata)
+        eq_(op.to_table(), schemacompare.CompareTable(t))
+        eq_(op.reverse().to_table(), schemacompare.CompareTable(t))
 
     def test_add_table(self):
         t = self.table
 
         op = ops.CreateTableOp.from_table(t)
-        is_(op.to_table(), t)
-        is_(op.reverse().to_table(), t)
-        is_(self.metadata, op.to_table().metadata)
+        eq_(op.to_table(), schemacompare.CompareTable(t))
+        eq_(op.reverse().to_table(), schemacompare.CompareTable(t))
 
     def test_drop_index(self):
         op = ops.DropIndexOp.from_index(self.ix)
-        is_(op.to_index(), self.ix)
-        is_(op.reverse().to_index(), self.ix)
+        eq_(op.to_index(), schemacompare.CompareIndex(self.ix))
+        eq_(op.reverse().to_index(), schemacompare.CompareIndex(self.ix))
 
     def test_create_index(self):
         op = ops.CreateIndexOp.from_index(self.ix)
-        is_(op.to_index(), self.ix)
-        is_(op.reverse().to_index(), self.ix)
+        eq_(op.to_index(), schemacompare.CompareIndex(self.ix))
+        eq_(op.reverse().to_index(), schemacompare.CompareIndex(self.ix))
 
 
 class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
@@ -1843,7 +1819,7 @@ class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
         c2 = Table("c2", m2c, Column("id", Integer, primary_key=True))
 
         diffs = self._fixture([m1a, m1b, m1c], [m2a, m2b, m2c])
-        eq_(diffs[0], ("add_table", c2))
+        eq_(diffs[0], ("add_table", schemacompare.CompareTable(c2)))
         eq_(diffs[1][0], "remove_table")
         eq_(diffs[1][1].name, "b2")
         eq_(diffs[2], ("add_column", None, "a", a.c.q))
@@ -1904,188 +1880,3 @@ class MultipleMetaDataTest(AutogenFixtureTest, TestBase):
             [m1a, m1b],
             [m2a, m2b],
         )
-
-
-class AutoincrementTest(AutogenFixtureTest, TestBase):
-    __backend__ = True
-    __requires__ = ("integer_subtype_comparisons",)
-
-    def test_alter_column_autoincrement_none(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table("a", m1, Column("x", Integer, nullable=False))
-        Table("a", m2, Column("x", Integer, nullable=True))
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        assert "autoincrement" not in ops.ops[0].ops[0].kw
-
-    def test_alter_column_autoincrement_pk_false(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("x", Integer, primary_key=True, autoincrement=False),
-        )
-        Table(
-            "a",
-            m2,
-            Column("x", BigInteger, primary_key=True, autoincrement=False),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], False)
-
-    def test_alter_column_autoincrement_pk_implicit_true(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table("a", m1, Column("x", Integer, primary_key=True))
-        Table("a", m2, Column("x", BigInteger, primary_key=True))
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], True)
-
-    def test_alter_column_autoincrement_pk_explicit_true(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a", m1, Column("x", Integer, primary_key=True, autoincrement=True)
-        )
-        Table(
-            "a",
-            m2,
-            Column("x", BigInteger, primary_key=True, autoincrement=True),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], True)
-
-    def test_alter_column_autoincrement_nonpk_false(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True),
-            Column("x", Integer, autoincrement=False),
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True),
-            Column("x", BigInteger, autoincrement=False),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], False)
-
-    def test_alter_column_autoincrement_nonpk_implicit_false(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True),
-            Column("x", Integer),
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True),
-            Column("x", BigInteger),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        assert "autoincrement" not in ops.ops[0].ops[0].kw
-
-    def test_alter_column_autoincrement_nonpk_explicit_true(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True, autoincrement=False),
-            Column("x", Integer, autoincrement=True),
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True, autoincrement=False),
-            Column("x", BigInteger, autoincrement=True),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], True)
-
-    def test_alter_column_autoincrement_compositepk_false(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True),
-            Column("x", Integer, primary_key=True, autoincrement=False),
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True),
-            Column("x", BigInteger, primary_key=True, autoincrement=False),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], False)
-
-    def test_alter_column_autoincrement_compositepk_implicit_false(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True),
-            Column("x", Integer, primary_key=True),
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True),
-            Column("x", BigInteger, primary_key=True),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        assert "autoincrement" not in ops.ops[0].ops[0].kw
-
-    @config.requirements.autoincrement_on_composite_pk
-    def test_alter_column_autoincrement_compositepk_explicit_true(self):
-        m1 = MetaData()
-        m2 = MetaData()
-
-        Table(
-            "a",
-            m1,
-            Column("id", Integer, primary_key=True, autoincrement=False),
-            Column("x", Integer, primary_key=True, autoincrement=True),
-            # on SQLA 1.0 and earlier, this being present
-            # trips the "add KEY for the primary key" so that the
-            # AUTO_INCREMENT keyword is accepted by MySQL.  SQLA 1.1 and
-            # greater the columns are just reorganized.
-            mysql_engine="InnoDB",
-        )
-        Table(
-            "a",
-            m2,
-            Column("id", Integer, primary_key=True, autoincrement=False),
-            Column("x", BigInteger, primary_key=True, autoincrement=True),
-        )
-
-        ops = self._fixture(m1, m2, return_ops=True)
-        is_(ops.ops[0].ops[0].kw["autoincrement"], True)
