@@ -1,5 +1,4 @@
 import re
-import sys
 
 import sqlalchemy as sa  # noqa
 from sqlalchemy import BigInteger
@@ -49,9 +48,6 @@ from alembic.testing import eq_ignore_whitespace
 from alembic.testing import mock
 from alembic.testing import TestBase
 from alembic.testing.fixtures import op_fixture
-from alembic.util import compat
-
-py3k = sys.version_info >= (3,)
 
 
 class AutogenRenderTest(TestBase):
@@ -185,7 +181,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_lower_code_idx', 'test', "
-            "[sa.text(!U'lower(code)')], unique=False)",
+            "[sa.text('lower(code)')], unique=False)",
         )
 
     def test_render_add_index_cast(self):
@@ -202,7 +198,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_lower_code_idx', 'test', "
-            "[sa.text(!U'CAST(code AS VARCHAR)')], unique=False)",
+            "[sa.text('CAST(code AS VARCHAR)')], unique=False)",
         )
 
     def test_render_add_index_desc(self):
@@ -218,7 +214,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_index('test_desc_code_idx', 'test', "
-            "[sa.text(!U'code DESC')], unique=False)",
+            "[sa.text('code DESC')], unique=False)",
         )
 
     def test_drop_index(self):
@@ -240,6 +236,23 @@ class AutogenRenderTest(TestBase):
             "op.drop_index('test_active_code_idx', table_name='test')",
         )
 
+    @testing.emits_warning("Can't validate argument ")
+    def test_render_drop_index_custom_kwarg(self):
+        t = Table(
+            "test",
+            MetaData(),
+            Column("id", Integer, primary_key=True),
+            Column("active", Boolean()),
+            Column("code", String(255)),
+        )
+        idx = Index(None, t.c.active, t.c.code, somedialect_foobar="option")
+        op_obj = ops.DropIndexOp.from_index(idx)
+        eq_ignore_whitespace(
+            autogenerate.render_op_text(self.autogen_context, op_obj),
+            "op.drop_index(op.f('ix_test_active'), table_name='test', "
+            "somedialect_foobar='option')",
+        )
+
     def test_drop_index_batch(self):
         """
         autogenerate.render._drop_index
@@ -815,7 +828,7 @@ class AutogenRenderTest(TestBase):
     def test_render_table_w_unicode_name(self):
         m = MetaData()
         t = Table(
-            compat.ue("\u0411\u0435\u0437"),
+            "\u0411\u0435\u0437",
             m,
             Column("id", Integer, primary_key=True),
         )
@@ -824,7 +837,7 @@ class AutogenRenderTest(TestBase):
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_table(%r,"
             "sa.Column('id', sa.Integer(), nullable=False),"
-            "sa.PrimaryKeyConstraint('id'))" % compat.ue("\u0411\u0435\u0437"),
+            "sa.PrimaryKeyConstraint('id'))" % "\u0411\u0435\u0437",
         )
 
     def test_render_table_w_unicode_schema(self):
@@ -833,7 +846,7 @@ class AutogenRenderTest(TestBase):
             "test",
             m,
             Column("id", Integer, primary_key=True),
-            schema=compat.ue("\u0411\u0435\u0437"),
+            schema="\u0411\u0435\u0437",
         )
         op_obj = ops.CreateTableOp.from_table(t)
         eq_ignore_whitespace(
@@ -841,7 +854,7 @@ class AutogenRenderTest(TestBase):
             "op.create_table('test',"
             "sa.Column('id', sa.Integer(), nullable=False),"
             "sa.PrimaryKeyConstraint('id'),"
-            "schema=%r)" % compat.ue("\u0411\u0435\u0437"),
+            "schema=%r)" % "\u0411\u0435\u0437",
         )
 
     def test_render_table_w_unsupported_constraint(self):
@@ -1176,7 +1189,7 @@ class AutogenRenderTest(TestBase):
         )
 
     def test_render_unicode_server_default(self):
-        default = compat.ue(
+        default = (
             "\u0411\u0435\u0437 "
             "\u043d\u0430\u0437\u0432\u0430\u043d\u0438\u044f"
         )
@@ -1244,6 +1257,7 @@ class AutogenRenderTest(TestBase):
 
         def render(type_, obj, context):
             if type_ == "foreign_key":
+                # causes it not to render
                 return None
             if type_ == "column":
                 if obj.name == "y":
@@ -1269,7 +1283,7 @@ class AutogenRenderTest(TestBase):
             Column("y", Integer),
             Column("q", MySpecialType()),
             PrimaryKeyConstraint("x"),
-            ForeignKeyConstraint(["x"], ["y"]),
+            ForeignKeyConstraint(["x"], ["remote.y"]),
         )
         op_obj = ops.CreateTableOp.from_table(t)
         result = autogenerate.render_op_text(self.autogen_context, op_obj)
@@ -1282,7 +1296,7 @@ class AutogenRenderTest(TestBase):
         )
         eq_(
             self.autogen_context.imports,
-            set(["from mypackage import MySpecialType"]),
+            {"from mypackage import MySpecialType"},
         )
 
     def test_render_modify_type(self):
@@ -1380,7 +1394,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], onupdate='CASCADE')",
@@ -1393,7 +1407,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], ondelete='CASCADE')",
@@ -1405,7 +1419,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], deferrable=True)",
@@ -1417,7 +1431,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], initially='XYZ')",
@@ -1435,7 +1449,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], "
@@ -1455,7 +1469,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )",
@@ -1474,7 +1488,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.nonexistent'], )",
@@ -1498,7 +1512,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )",
@@ -1517,7 +1531,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['t2.c_rem'], )",
@@ -1537,7 +1551,7 @@ class AutogenRenderTest(TestBase):
 
         eq_ignore_whitespace(
             autogenerate.render._render_constraint(
-                const, self.autogen_context
+                const, self.autogen_context, m
             ),
             "sa.ForeignKeyConstraint(['c_rem'], ['t.c'], "
             "name='fk1', use_alter=True)",
@@ -1555,7 +1569,7 @@ class AutogenRenderTest(TestBase):
                 r"u'",
                 "'",
                 autogenerate.render._render_constraint(
-                    fk, self.autogen_context
+                    fk, self.autogen_context, m
                 ),
             ),
             "sa.ForeignKeyConstraint(['c'], ['foo.t2.c_rem'], "
@@ -1567,8 +1581,9 @@ class AutogenRenderTest(TestBase):
             autogenerate.render._render_check_constraint(
                 CheckConstraint("im a constraint", name="cc1"),
                 self.autogen_context,
+                None,
             ),
-            "sa.CheckConstraint(!U'im a constraint', name='cc1')",
+            "sa.CheckConstraint('im a constraint', name='cc1')",
         )
 
     def test_render_check_constraint_sqlexpr(self):
@@ -1577,18 +1592,22 @@ class AutogenRenderTest(TestBase):
         ten = literal_column("10")
         eq_ignore_whitespace(
             autogenerate.render._render_check_constraint(
-                CheckConstraint(and_(c > five, c < ten)), self.autogen_context
+                CheckConstraint(and_(c > five, c < ten)),
+                self.autogen_context,
+                None,
             ),
-            "sa.CheckConstraint(!U'c > 5 AND c < 10')",
+            "sa.CheckConstraint('c > 5 AND c < 10')",
         )
 
     def test_render_check_constraint_literal_binds(self):
         c = column("c")
         eq_ignore_whitespace(
             autogenerate.render._render_check_constraint(
-                CheckConstraint(and_(c > 5, c < 10)), self.autogen_context
+                CheckConstraint(and_(c > 5, c < 10)),
+                self.autogen_context,
+                None,
             ),
-            "sa.CheckConstraint(!U'c > 5 AND c < 10')",
+            "sa.CheckConstraint('c > 5 AND c < 10')",
         )
 
     def test_render_unique_constraint_opts(self):
@@ -1598,6 +1617,7 @@ class AutogenRenderTest(TestBase):
             autogenerate.render._render_unique_constraint(
                 UniqueConstraint(t.c.c, name="uq_1", deferrable="XYZ"),
                 self.autogen_context,
+                None,
             ),
             "sa.UniqueConstraint('c', deferrable='XYZ', name='uq_1')",
         )
@@ -1608,13 +1628,13 @@ class AutogenRenderTest(TestBase):
             "t",
             m,
             Column("c", Integer),
-            schema=compat.ue("\u0411\u0435\u0437"),
+            schema="\u0411\u0435\u0437",
         )
         op_obj = ops.AddConstraintOp.from_constraint(UniqueConstraint(t.c.c))
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_unique_constraint(None, 't', ['c'], "
-            "schema=%r)" % compat.ue("\u0411\u0435\u0437"),
+            "schema=%r)" % "\u0411\u0435\u0437",
         )
 
     def test_render_modify_nullable_w_default(self):
@@ -1712,7 +1732,7 @@ class AutogenRenderTest(TestBase):
             "# ### commands auto generated by Alembic - please adjust! ###\n"
             "    op.create_table('sometable',\n"
             "    sa.Column('x', sa.DateTime(), "
-            "server_default=sa.text(!U'now()'), nullable=True)\n"
+            "server_default=sa.text('now()'), nullable=True)\n"
             "    )\n"
             "    # ### end Alembic commands ###",
         )
@@ -1732,7 +1752,7 @@ class AutogenRenderTest(TestBase):
             "# ### commands auto generated by Alembic - please adjust! ###\n"
             "    op.create_table('sometable',\n"
             "    sa.Column('x', sa.DateTime(), "
-            "server_default=sa.text(!U'(CURRENT_TIMESTAMP)'), nullable=True)\n"
+            "server_default=sa.text('(CURRENT_TIMESTAMP)'), nullable=True)\n"
             "    )\n"
             "    # ### end Alembic commands ###",
         )
@@ -1813,7 +1833,7 @@ class AutogenRenderTest(TestBase):
         )
         eq_(
             self.autogen_context.imports,
-            set(["from sqlalchemy.dialects import mysql"]),
+            {"from sqlalchemy.dialects import mysql"},
         )
 
     def test_render_server_default_text(self):
@@ -1827,7 +1847,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             result,
             "sa.Column('updated_at', sa.TIMESTAMP(), "
-            "server_default=sa.text(!U'now()'), "
+            "server_default=sa.text('now()'), "
             "nullable=False)",
         )
 
@@ -1840,7 +1860,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             result,
             "sa.Column('updated_at', sa.Boolean(), "
-            "server_default=sa.text(!U'0'), "
+            "server_default=sa.text('0'), "
             "nullable=False)",
         )
 
@@ -1855,7 +1875,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             result,
             "sa.Column('updated_at', sa.TIMESTAMP(), "
-            "server_default=sa.text(!U'now()'), "
+            "server_default=sa.text('now()'), "
             "nullable=False)",
         )
 
@@ -1880,7 +1900,7 @@ class AutogenRenderTest(TestBase):
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.alter_column('sometable', 'somecolumn', "
             "existing_type=sa.Integer(), nullable=True, "
-            "existing_server_default=sa.text(!U'5'))",
+            "existing_server_default=sa.text('5'))",
         )
 
     def test_render_executesql_plaintext(self):
@@ -2052,7 +2072,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.add_column('foo', sa.Column('x', sa.Integer(), "
-            "sa.Computed(!U'5', ), nullable=True))",
+            "sa.Computed('5', ), nullable=True))",
         )
 
     @config.requirements.computed_columns_api
@@ -2064,7 +2084,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.add_column('foo', sa.Column('x', sa.Integer(), "
-            "sa.Computed(!U'5', persisted=%s), nullable=True))" % persisted,
+            "sa.Computed('5', persisted=%s), nullable=True))" % persisted,
         )
 
     @config.requirements.computed_columns_api
@@ -2075,7 +2095,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.alter_column('sometable', 'somecolumn', "
-            "server_default=sa.Computed(!U'7', ))",
+            "server_default=sa.Computed('7', ))",
         )
 
     @config.requirements.computed_columns_api
@@ -2088,7 +2108,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.alter_column('sometable', 'somecolumn', "
-            "existing_server_default=sa.Computed(!U'42', ))",
+            "existing_server_default=sa.Computed('42', ))",
         )
 
     @config.requirements.computed_columns_api
@@ -2104,7 +2124,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.alter_column('sometable', 'somecolumn', server_default"
-            "=sa.Computed(!U'7', persisted=%s))" % persisted,
+            "=sa.Computed('7', persisted=%s))" % persisted,
         )
 
     @config.requirements.computed_columns_api
@@ -2119,7 +2139,7 @@ class AutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.alter_column('sometable', 'somecolumn', "
-            "existing_server_default=sa.Computed(!U'42', persisted=%s))"
+            "existing_server_default=sa.Computed('42', persisted=%s))"
             % persisted,
         )
 
@@ -2248,7 +2268,9 @@ class RenderNamingConventionTest(TestBase):
         t = Table("t", self.metadata, Column("c", Integer))
         eq_ignore_whitespace(
             autogenerate.render._render_unique_constraint(
-                UniqueConstraint(t.c.c, deferrable="XYZ"), self.autogen_context
+                UniqueConstraint(t.c.c, deferrable="XYZ"),
+                self.autogen_context,
+                None,
             ),
             "sa.UniqueConstraint('c', deferrable='XYZ', "
             "name=op.f('uq_ct_t_c'))",
@@ -2258,7 +2280,7 @@ class RenderNamingConventionTest(TestBase):
         t = Table("t", self.metadata, Column("c", Integer))
         eq_ignore_whitespace(
             autogenerate.render._render_unique_constraint(
-                UniqueConstraint(t.c.c, name="q"), self.autogen_context
+                UniqueConstraint(t.c.c, name="q"), self.autogen_context, None
             ),
             "sa.UniqueConstraint('c', name='q')",
         )
@@ -2316,7 +2338,7 @@ class RenderNamingConventionTest(TestBase):
         uq = [c for c in t.constraints if isinstance(c, UniqueConstraint)][0]
         eq_ignore_whitespace(
             autogenerate.render._render_unique_constraint(
-                uq, self.autogen_context
+                uq, self.autogen_context, None
             ),
             "sa.UniqueConstraint('c', name=op.f('uq_ct_t_c'))",
         )
@@ -2338,7 +2360,7 @@ class RenderNamingConventionTest(TestBase):
         eq_ignore_whitespace(
             autogenerate.render_op_text(self.autogen_context, op_obj),
             "op.create_table('t',sa.Column('c', sa.Integer(), nullable=True),"
-            "sa.CheckConstraint(!U'c > 5', name=op.f('ck_ct_t')))",
+            "sa.CheckConstraint('c > 5', name=op.f('ck_ct_t')))",
         )
 
     def test_inline_fk(self):
@@ -2370,9 +2392,9 @@ class RenderNamingConventionTest(TestBase):
 
         eq_ignore_whitespace(
             autogenerate.render._render_check_constraint(
-                ck, self.autogen_context
+                ck, self.autogen_context, None
             ),
-            "sa.CheckConstraint(!U'im a constraint', name=op.f('ck_t_cc1'))",
+            "sa.CheckConstraint('im a constraint', name=op.f('ck_t_cc1'))",
         )
 
     def test_create_table_plus_add_index_in_modify(self):
