@@ -2,6 +2,7 @@ from sqlalchemy import BigInteger
 from sqlalchemy import Boolean
 from sqlalchemy import Column
 from sqlalchemy import DateTime
+from sqlalchemy import exc
 from sqlalchemy import Float
 from sqlalchemy import func
 from sqlalchemy import Index
@@ -15,6 +16,7 @@ from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import text
 from sqlalchemy import types
+from sqlalchemy import UniqueConstraint
 from sqlalchemy.dialects.postgresql import ARRAY
 from sqlalchemy.dialects.postgresql import BYTEA
 from sqlalchemy.dialects.postgresql import HSTORE
@@ -34,9 +36,11 @@ from alembic.autogenerate.compare import _compare_server_default
 from alembic.autogenerate.compare import _compare_tables
 from alembic.autogenerate.compare import _render_server_default_for_compare
 from alembic.migration import MigrationContext
-from alembic.operations import Operations
 from alembic.operations import ops
 from alembic.script import ScriptDirectory
+from alembic.testing import assert_raises_message
+from alembic.testing import assertions
+from alembic.testing import combinations
 from alembic.testing import config
 from alembic.testing import eq_
 from alembic.testing import eq_ignore_whitespace
@@ -46,8 +50,11 @@ from alembic.testing.env import clear_staging_env
 from alembic.testing.env import staging_env
 from alembic.testing.env import write_script
 from alembic.testing.fixtures import capture_context_buffer
+from alembic.testing.fixtures import FutureEngineMixin
 from alembic.testing.fixtures import op_fixture
+from alembic.testing.fixtures import TablesTest
 from alembic.testing.fixtures import TestBase
+from alembic.testing.suite._autogen_fixtures import AutogenFixtureTest
 from alembic.util import sqla_compat
 
 
@@ -100,6 +107,19 @@ class PostgresqlOpTest(TestBase):
             "CREATE INDEX CONCURRENTLY geocoded ON locations (coordinates)"
         )
 
+    @config.requirements.sqlalchemy_14
+    def test_create_index_postgresql_include(self):
+        context = op_fixture("postgresql")
+        op.create_index(
+            "i", "t", ["c1", "c2"], unique=False, postgresql_include=["inc"]
+        )
+        context.assert_("CREATE INDEX i ON t (c1, c2) INCLUDE (inc)")
+
+    def test_create_index_postgresql_include_is_none(self):
+        context = op_fixture("postgresql")
+        op.create_index("i", "t", ["c1", "c2"], unique=False)
+        context.assert_("CREATE INDEX i ON t (c1, c2)")
+
     def test_drop_index_postgresql_concurrently(self):
         context = op_fixture("postgresql")
         op.drop_index("geocoded", "locations", postgresql_concurrently=True)
@@ -155,7 +175,6 @@ class PostgresqlOpTest(TestBase):
             'USING gist ("SomeColumn" WITH >) WHERE ("SomeColumn" > 5)'
         )
 
-    @config.requirements.comments_api
     def test_add_column_with_comment(self):
         context = op_fixture("postgresql")
         op.add_column("t", Column("q", Integer, comment="This is a comment"))
@@ -164,7 +183,6 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON COLUMN t.q IS 'This is a comment'",
         )
 
-    @config.requirements.comments_api
     def test_alter_column_with_comment(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -181,7 +199,6 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON COLUMN foo.t.c IS 'This is a column comment'",
         )
 
-    @config.requirements.comments_api
     def test_alter_column_add_comment(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -196,7 +213,6 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON COLUMN foo.t.c IS 'This is a column comment'"
         )
 
-    @config.requirements.comments_api
     def test_alter_column_add_comment_table_and_column_quoting(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -211,7 +227,6 @@ class PostgresqlOpTest(TestBase):
             'COMMENT ON COLUMN foo."T"."C" IS \'This is a column comment\''
         )
 
-    @config.requirements.comments_api
     def test_alter_column_add_comment_quoting(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -226,7 +241,6 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON COLUMN foo.t.c IS 'This is a column ''comment'''"
         )
 
-    @config.requirements.comments_api
     def test_alter_column_drop_comment(self):
         context = op_fixture("postgresql")
         op.alter_column(
@@ -240,7 +254,6 @@ class PostgresqlOpTest(TestBase):
 
         context.assert_("COMMENT ON COLUMN foo.t.c IS NULL")
 
-    @config.requirements.comments_api
     def test_create_table_with_comment(self):
         context = op_fixture("postgresql")
         op.create_table(
@@ -255,7 +268,6 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON TABLE t2 IS 't2 comment'",
         )
 
-    @config.requirements.comments_api
     def test_create_table_with_column_comments(self):
         context = op_fixture("postgresql")
         op.create_table(
@@ -272,21 +284,19 @@ class PostgresqlOpTest(TestBase):
             "COMMENT ON COLUMN t2.c2 IS 'c2 comment'",
         )
 
-    @config.requirements.comments_api
     def test_create_table_comment(self):
         # this is handled by SQLAlchemy's compilers
         context = op_fixture("postgresql")
         op.create_table_comment("t2", comment="t2 table", schema="foo")
         context.assert_("COMMENT ON TABLE foo.t2 IS 't2 table'")
 
-    @config.requirements.comments_api
     def test_drop_table_comment(self):
         # this is handled by SQLAlchemy's compilers
         context = op_fixture("postgresql")
         op.drop_table_comment("t2", existing_comment="t2 table", schema="foo")
         context.assert_("COMMENT ON TABLE foo.t2 IS NULL")
 
-    @config.requirements.computed_columns_api
+    @config.requirements.computed_columns
     def test_add_column_computed(self):
         context = op_fixture("postgresql")
         op.add_column(
@@ -298,6 +308,134 @@ class PostgresqlOpTest(TestBase):
             "INTEGER GENERATED ALWAYS AS (foo * 5) STORED"
         )
 
+    @combinations(
+        (lambda: sqla_compat.Computed("foo * 5"), lambda: None),
+        (lambda: None, lambda: sqla_compat.Computed("foo * 5")),
+        (
+            lambda: sqla_compat.Computed("foo * 42"),
+            lambda: sqla_compat.Computed("foo * 5"),
+        ),
+    )
+    @config.requirements.computed_columns
+    def test_alter_column_computed_not_supported(self, sd, esd):
+        op_fixture("postgresql")
+        assert_raises_message(
+            exc.CompileError,
+            'Adding or removing a "computed" construct, e.g. '
+            "GENERATED ALWAYS AS, to or from an existing column is not "
+            "supported.",
+            op.alter_column,
+            "t1",
+            "c1",
+            server_default=sd(),
+            existing_server_default=esd(),
+        )
+
+    @config.requirements.identity_columns
+    @combinations(
+        ({}, None),
+        (dict(always=True), None),
+        (
+            dict(start=3, increment=33, maxvalue=99, cycle=True),
+            "INCREMENT BY 33 START WITH 3 MAXVALUE 99 CYCLE",
+        ),
+    )
+    def test_add_column_identity(self, kw, text):
+        context = op_fixture("postgresql")
+        op.add_column(
+            "t1",
+            Column("some_column", Integer, sqla_compat.Identity(**kw)),
+        )
+        qualification = "ALWAYS" if kw.get("always", False) else "BY DEFAULT"
+        options = " (%s)" % text if text else ""
+        context.assert_(
+            "ALTER TABLE t1 ADD COLUMN some_column "
+            "INTEGER GENERATED %s AS IDENTITY%s" % (qualification, options)
+        )
+
+    @config.requirements.identity_columns
+    @combinations(
+        ({}, None),
+        (dict(always=True), None),
+        (
+            dict(start=3, increment=33, maxvalue=99, cycle=True),
+            "INCREMENT BY 33 START WITH 3 MAXVALUE 99 CYCLE",
+        ),
+    )
+    def test_add_identity_to_column(self, kw, text):
+        context = op_fixture("postgresql")
+        op.alter_column(
+            "t1",
+            "some_column",
+            server_default=sqla_compat.Identity(**kw),
+            existing_server_default=None,
+        )
+        qualification = "ALWAYS" if kw.get("always", False) else "BY DEFAULT"
+        options = " (%s)" % text if text else ""
+        context.assert_(
+            "ALTER TABLE t1 ALTER COLUMN some_column ADD "
+            "GENERATED %s AS IDENTITY%s" % (qualification, options)
+        )
+
+    @config.requirements.identity_columns
+    def test_remove_identity_from_column(self):
+        context = op_fixture("postgresql")
+        op.alter_column(
+            "t1",
+            "some_column",
+            server_default=None,
+            existing_server_default=sqla_compat.Identity(),
+        )
+        context.assert_(
+            "ALTER TABLE t1 ALTER COLUMN some_column DROP IDENTITY"
+        )
+
+    @config.requirements.identity_columns
+    @combinations(
+        ({}, dict(always=True), "SET GENERATED ALWAYS"),
+        (
+            dict(always=True),
+            dict(always=False, start=3),
+            "SET GENERATED BY DEFAULT SET START WITH 3",
+        ),
+        (
+            dict(always=True, start=3, increment=2, minvalue=-3, maxvalue=99),
+            dict(
+                always=True,
+                start=3,
+                increment=1,
+                minvalue=-3,
+                maxvalue=99,
+                cycle=True,
+            ),
+            "SET CYCLE SET INCREMENT BY 1",
+        ),
+        (
+            dict(
+                always=False,
+                start=3,
+                maxvalue=9999,
+                minvalue=0,
+            ),
+            dict(always=False, start=3, order=True, on_null=False, cache=2),
+            "SET CACHE 2",
+        ),
+        (
+            dict(always=False),
+            dict(always=None, minvalue=0),
+            "SET MINVALUE 0",
+        ),
+    )
+    def test_change_identity_in_column(self, existing, updated, text):
+        context = op_fixture("postgresql")
+        op.alter_column(
+            "t1",
+            "some_column",
+            server_default=sqla_compat.Identity(**updated),
+            existing_server_default=sqla_compat.Identity(**existing),
+        )
+        context.assert_("ALTER TABLE t1 ALTER COLUMN some_column %s" % text)
+
 
 class PGAutocommitBlockTest(TestBase):
     __only_on__ = "postgresql"
@@ -307,17 +445,24 @@ class PGAutocommitBlockTest(TestBase):
         self.conn = conn = config.db.connect()
 
         with conn.begin():
-            conn.execute("CREATE TYPE mood AS ENUM ('sad', 'ok', 'happy');")
+            conn.execute(
+                text("CREATE TYPE mood AS ENUM ('sad', 'ok', 'happy')")
+            )
 
     def tearDown(self):
         with self.conn.begin():
-            self.conn.execute("DROP TYPE mood")
+            self.conn.execute(text("DROP TYPE mood"))
 
-    def test_alter_enum(self):
-        context = MigrationContext.configure(connection=self.conn)
-        with context.begin_transaction(_per_migration=True):
-            with context.autocommit_block():
-                context.execute("ALTER TYPE mood ADD VALUE 'soso'")
+    def test_alter_enum(self, migration_context):
+        with migration_context.begin_transaction(_per_migration=True):
+            with migration_context.autocommit_block():
+                migration_context.execute(
+                    text("ALTER TYPE mood ADD VALUE 'soso'")
+                )
+
+
+class PGAutocommitBlockTestFuture(FutureEngineMixin, PGAutocommitBlockTest):
+    pass
 
 
 class PGOfflineEnumTest(TestBase):
@@ -423,53 +568,39 @@ def downgrade():
         assert "DROP TYPE pgenum" in buf.getvalue()
 
 
-class PostgresqlInlineLiteralTest(TestBase):
+class PostgresqlInlineLiteralTest(TablesTest):
     __only_on__ = "postgresql"
     __backend__ = True
 
     @classmethod
-    def setup_class(cls):
-        cls.bind = config.db
-        cls.bind.execute(
-            """
-            create table tab (
-                col varchar(50)
-            )
-        """
-        )
-        cls.bind.execute(
-            """
-            insert into tab (col) values
-                ('old data 1'),
-                ('old data 2.1'),
-                ('old data 3')
-        """
-        )
+    def define_tables(cls, metadata):
+        Table("tab", metadata, Column("col", String(50)))
 
     @classmethod
-    def teardown_class(cls):
-        cls.bind.execute("drop table tab")
+    def insert_data(cls, connection):
+        connection.execute(
+            text(
+                """
+                insert into tab (col) values
+                    ('old data 1'),
+                    ('old data 2.1'),
+                    ('old data 3')
+            """
+            )
+        )
 
-    def setUp(self):
-        self.conn = self.bind.connect()
-        ctx = MigrationContext.configure(self.conn)
-        self.op = Operations(ctx)
-
-    def tearDown(self):
-        self.conn.close()
-
-    def test_inline_percent(self):
+    def test_inline_percent(self, connection, ops_context):
         # TODO: here's the issue, you need to escape this.
         tab = table("tab", column("col"))
-        self.op.execute(
+        ops_context.execute(
             tab.update()
-            .where(tab.c.col.like(self.op.inline_literal("%.%")))
-            .values(col=self.op.inline_literal("new data")),
+            .where(tab.c.col.like(ops_context.inline_literal("%.%")))
+            .values(col=ops_context.inline_literal("new data")),
             execution_options={"no_parameters": True},
         )
         eq_(
-            self.conn.execute(
-                "select count(*) from tab where col='new data'"
+            connection.execute(
+                text("select count(*) from tab where col='new data'")
             ).scalar(),
             1,
         )
@@ -489,7 +620,7 @@ class PostgresqlDefaultCompareTest(TestBase):
         )
 
     def setUp(self):
-        self.metadata = MetaData(self.bind)
+        self.metadata = MetaData()
         self.autogen_context = api.AutogenContext(self.migration_context)
 
     @classmethod
@@ -497,7 +628,8 @@ class PostgresqlDefaultCompareTest(TestBase):
         clear_staging_env()
 
     def tearDown(self):
-        self.metadata.drop_all()
+        with config.db.begin() as conn:
+            self.metadata.drop_all(conn)
 
     def _compare_default_roundtrip(
         self, type_, orig_default, alternate=None, diff_expected=None
@@ -636,7 +768,7 @@ class PostgresqlDefaultCompareTest(TestBase):
         )
 
     def test_compare_unicode_literal(self):
-        self._compare_default_roundtrip(String(), u"im a default")
+        self._compare_default_roundtrip(String(), "im a default")
 
     # TOOD: will need to actually eval() the repr() and
     # spend more effort figuring out exactly the kind of expression
@@ -706,9 +838,7 @@ class PostgresqlDetectSerialTest(TestBase):
         insp = inspect(config.db)
 
         uo = ops.UpgradeOps(ops=[])
-        _compare_tables(
-            set([(None, "t")]), set([]), insp, uo, self.autogen_context
-        )
+        _compare_tables({(None, "t")}, set(), insp, uo, self.autogen_context)
         diffs = uo.as_diffs()
         tab = diffs[0][1]
 
@@ -725,8 +855,8 @@ class PostgresqlDetectSerialTest(TestBase):
         Table("t", m2, Column("x", BigInteger()))
         self.autogen_context.metadata = m2
         _compare_tables(
-            set([(None, "t")]),
-            set([(None, "t")]),
+            {(None, "t")},
+            {(None, "t")},
             insp,
             uo,
             self.autogen_context,
@@ -801,7 +931,7 @@ class PostgresqlAutogenRenderTest(TestBase):
             autogenerate.render_op_text(autogen_context, op_obj),
             """op.create_index('foo_idx', 't', \
 ['x', 'y'], unique=False, """
-            """postgresql_where=sa.text(!U"y = 'something'"))""",
+            """postgresql_where=sa.text("y = 'something'"))""",
         )
 
     def test_render_server_default_native_boolean(self):
@@ -812,7 +942,7 @@ class PostgresqlAutogenRenderTest(TestBase):
         eq_ignore_whitespace(
             result,
             "sa.Column('updated_at', sa.Boolean(), "
-            "server_default=sa.text(!U'false'), "
+            "server_default=sa.text('false'), "
             "nullable=False)",
         )
 
@@ -938,7 +1068,7 @@ class PostgresqlAutogenRenderTest(TestBase):
             autogenerate.render_op_text(autogen_context, op_obj),
             "op.create_exclude_constraint('t_excl_x', "
             "'t', (sa.column('x'), '>'), "
-            "where=sa.text(!U'x != 2'), using='gist')",
+            "where=sa.text('x != 2'), using='gist')",
         )
 
     def test_add_exclude_constraint_case_sensitive(self):
@@ -964,7 +1094,7 @@ class PostgresqlAutogenRenderTest(TestBase):
             autogenerate.render_op_text(autogen_context, op_obj),
             "op.create_exclude_constraint('t_excl_x', 'TTAble', "
             "(sa.column('XColumn'), '>'), "
-            "where=sa.text(!U'\"XColumn\" != 2'), using='gist')",
+            "where=sa.text('\"XColumn\" != 2'), using='gist')",
         )
 
     def test_inline_exclude_constraint(self):
@@ -993,7 +1123,7 @@ class PostgresqlAutogenRenderTest(TestBase):
             "op.create_table('t',sa.Column('x', sa.String(), nullable=True),"
             "sa.Column('y', sa.String(), nullable=True),"
             "postgresql.ExcludeConstraint((sa.column('x'), '>'), "
-            "where=sa.text(!U'x != 2'), using='gist', name='t_excl_x')"
+            "where=sa.text('x != 2'), using='gist', name='t_excl_x')"
             ")",
         )
 
@@ -1021,7 +1151,7 @@ class PostgresqlAutogenRenderTest(TestBase):
             "nullable=True),"
             "sa.Column('YColumn', sa.String(), nullable=True),"
             "postgresql.ExcludeConstraint((sa.column('XColumn'), '>'), "
-            "where=sa.text(!U'\"XColumn\" != 2'), using='gist', "
+            "where=sa.text('\"XColumn\" != 2'), using='gist', "
             "name='TExclX'))",
         )
 
@@ -1036,3 +1166,203 @@ class PostgresqlAutogenRenderTest(TestBase):
             autogenerate.render._repr_type(JSONB(), self.autogen_context),
             "postgresql.JSONB(astext_type=sa.Text())",
         )
+
+
+class PGUniqueIndexAutogenerateTest(AutogenFixtureTest, TestBase):
+    __only_on__ = "postgresql"
+    __backend__ = True
+
+    def test_idx_added_schema(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table("add_ix", m1, Column("x", String(50)), schema="test_schema")
+        Table(
+            "add_ix",
+            m2,
+            Column("x", String(50)),
+            Index("ix_1", "x"),
+            schema="test_schema",
+        )
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs[0][0], "add_index")
+        eq_(diffs[0][1].name, "ix_1")
+
+    def test_idx_unchanged_schema(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table(
+            "add_ix",
+            m1,
+            Column("x", String(50)),
+            Index("ix_1", "x"),
+            schema="test_schema",
+        )
+        Table(
+            "add_ix",
+            m2,
+            Column("x", String(50)),
+            Index("ix_1", "x"),
+            schema="test_schema",
+        )
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs, [])
+
+    def test_uq_added_schema(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table("add_uq", m1, Column("x", String(50)), schema="test_schema")
+        Table(
+            "add_uq",
+            m2,
+            Column("x", String(50)),
+            UniqueConstraint("x", name="ix_1"),
+            schema="test_schema",
+        )
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs[0][0], "add_constraint")
+        eq_(diffs[0][1].name, "ix_1")
+
+    def test_uq_unchanged_schema(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table(
+            "add_uq",
+            m1,
+            Column("x", String(50)),
+            UniqueConstraint("x", name="ix_1"),
+            schema="test_schema",
+        )
+        Table(
+            "add_uq",
+            m2,
+            Column("x", String(50)),
+            UniqueConstraint("x", name="ix_1"),
+            schema="test_schema",
+        )
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs, [])
+
+    @config.requirements.btree_gist
+    def test_exclude_const_unchanged(self):
+        from sqlalchemy.dialects.postgresql import TSRANGE, ExcludeConstraint
+
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table(
+            "add_excl",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("period", TSRANGE),
+            ExcludeConstraint(("period", "&&"), name="quarters_period_excl"),
+        )
+
+        Table(
+            "add_excl",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("period", TSRANGE),
+            ExcludeConstraint(("period", "&&"), name="quarters_period_excl"),
+        )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_same_tname_two_schemas(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table("add_ix", m1, Column("x", String(50)), Index("ix_1", "x"))
+
+        Table("add_ix", m2, Column("x", String(50)), Index("ix_1", "x"))
+        Table("add_ix", m2, Column("x", String(50)), schema="test_schema")
+
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs[0][0], "add_table")
+        eq_(len(diffs), 1)
+
+    def test_uq_dropped(self):
+        m1 = MetaData()
+        m2 = MetaData()
+        Table(
+            "add_uq",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("name", String),
+            UniqueConstraint("name", name="uq_name"),
+        )
+        Table(
+            "add_uq",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("name", String),
+        )
+        diffs = self._fixture(m1, m2, include_schemas=True)
+        eq_(diffs[0][0], "remove_constraint")
+        eq_(diffs[0][1].name, "uq_name")
+        eq_(len(diffs), 1)
+
+    def _functional_index_warn(self):
+        return (r"Skip.*refl",)
+
+    def test_functional_ix_one(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        t1 = Table(
+            "foo",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("email", String(50)),
+        )
+        Index("email_idx", func.lower(t1.c.email), unique=True)
+
+        t2 = Table(
+            "foo",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("email", String(50)),
+        )
+        Index("email_idx", func.lower(t2.c.email), unique=True)
+
+        with assertions.expect_warnings(*self._functional_index_warn()):
+            diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_functional_ix_two(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        t1 = Table(
+            "foo",
+            m1,
+            Column("id", Integer, primary_key=True),
+            Column("email", String(50)),
+            Column("name", String(50)),
+        )
+        Index(
+            "email_idx",
+            func.coalesce(t1.c.email, t1.c.name).desc(),
+            unique=True,
+        )
+
+        t2 = Table(
+            "foo",
+            m2,
+            Column("id", Integer, primary_key=True),
+            Column("email", String(50)),
+            Column("name", String(50)),
+        )
+        Index(
+            "email_idx",
+            func.coalesce(t2.c.email, t2.c.name).desc(),
+            unique=True,
+        )
+
+        with assertions.expect_warnings(*self._functional_index_warn()):
+            diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
