@@ -1,13 +1,15 @@
 import datetime
 import os
 import re
+from unittest.mock import patch
 
 from dateutil import tz
 import sqlalchemy as sa
-from sqlalchemy.engine.reflection import Inspector
+from sqlalchemy import inspect
 
 from alembic import autogenerate
 from alembic import command
+from alembic import testing
 from alembic import util
 from alembic.environment import EnvironmentContext
 from alembic.operations import ops
@@ -15,6 +17,7 @@ from alembic.script import ScriptDirectory
 from alembic.testing import assert_raises_message
 from alembic.testing import assertions
 from alembic.testing import eq_
+from alembic.testing import expect_raises_message
 from alembic.testing import is_
 from alembic.testing import mock
 from alembic.testing import ne_
@@ -56,7 +59,7 @@ class GeneralOrderedTests(TestBase):
         self._test_008_long_name_configurable()
 
     def _test_001_environment(self):
-        assert_set = set(["env.py", "script.py.mako", "README"])
+        assert_set = {"env.py", "script.py.mako", "README"}
         eq_(assert_set.intersection(os.listdir(env.dir)), assert_set)
 
     def _test_002_rev_ids(self):
@@ -95,7 +98,7 @@ class GeneralOrderedTests(TestBase):
         )
         eq_(script.revision, def_)
         eq_(script.down_revision, abc)
-        eq_(env.get_revision(abc).nextrev, set([def_]))
+        eq_(env.get_revision(abc).nextrev, {def_})
         assert script.module.down_revision == abc
         assert callable(script.module.upgrade)
         assert callable(script.module.downgrade)
@@ -109,7 +112,7 @@ class GeneralOrderedTests(TestBase):
         env = staging_env(create=False)
         abc_rev = env.get_revision(abc)
         def_rev = env.get_revision(def_)
-        eq_(abc_rev.nextrev, set([def_]))
+        eq_(abc_rev.nextrev, {def_})
         eq_(abc_rev.revision, abc)
         eq_(def_rev.down_revision, abc)
         eq_(env.get_heads(), [def_])
@@ -180,6 +183,33 @@ class ScriptNamingTest(TestBase):
             ),
         )
 
+    @testing.combinations(
+        (
+            datetime.datetime(2012, 7, 25, 15, 8, 5, tzinfo=tz.gettz("UTC")),
+            "%s/versions/1343228885_12345_this_is_a_"
+            "message_2012_7_25_15_8_5.py",
+        ),
+        (
+            datetime.datetime(2012, 7, 25, 15, 8, 6, tzinfo=tz.gettz("UTC")),
+            "%s/versions/1343228886_12345_this_is_a_"
+            "message_2012_7_25_15_8_6.py",
+        ),
+    )
+    def test_epoch(self, create_date, expected):
+        script = ScriptDirectory(
+            _get_staging_directory(),
+            file_template="%(epoch)s_%(rev)s_%(slug)s_"
+            "%(year)s_%(month)s_"
+            "%(day)s_%(hour)s_"
+            "%(minute)s_%(second)s",
+        )
+        eq_(
+            script._rev_path(
+                script.versions, "12345", "this is a message", create_date
+            ),
+            os.path.abspath(expected % _get_staging_directory()),
+        )
+
     def _test_tz(self, timezone_arg, given, expected):
         script = ScriptDirectory(
             _get_staging_directory(),
@@ -250,6 +280,17 @@ class ScriptNamingTest(TestBase):
             datetime.datetime(2012, 7, 25, 15, 8, 5),
         )
 
+    def test_no_dateutil_module(self):
+        with patch("alembic.script.base.tz", new=None):
+            with expect_raises_message(
+                CommandError, "The library 'python-dateutil' is required"
+            ):
+                self._test_tz(
+                    "utc",
+                    datetime.datetime(2012, 7, 25, 15, 8, 5),
+                    datetime.datetime(2012, 7, 25, 15, 8, 5),
+                )
+
 
 class RevisionCommandTest(TestBase):
     def setUp(self):
@@ -275,7 +316,7 @@ class RevisionCommandTest(TestBase):
         rev = script.get_revision(rev.revision)
         eq_(rev.down_revision, self.b)
         assert "some message" in rev.doc
-        eq_(set(script.get_heads()), set([rev.revision, self.c]))
+        eq_(set(script.get_heads()), {rev.revision, self.c})
 
     def test_create_script_missing_splice(self):
         assert_raises_message(
@@ -555,10 +596,7 @@ def downgrade():
         with self._env_fixture(process_revision_directives, m):
             command.upgrade(self.cfg, "heads")
 
-            eq_(
-                Inspector.from_engine(self.engine).get_table_names(),
-                ["alembic_version"],
-            )
+            eq_(inspect(self.engine).get_table_names(), ["alembic_version"])
 
             command.revision(
                 self.cfg, message="some message", autogenerate=True
@@ -567,16 +605,13 @@ def downgrade():
             command.upgrade(self.cfg, "model1@head")
 
             eq_(
-                Inspector.from_engine(self.engine).get_table_names(),
+                inspect(self.engine).get_table_names(),
                 ["alembic_version", "t"],
             )
 
             command.upgrade(self.cfg, "model2@head")
 
-            eq_(
-                Inspector.from_engine(self.engine).get_table_names(),
-                ["alembic_version"],
-            )
+            eq_(inspect(self.engine).get_table_names(), ["alembic_version"])
 
     def test_programmatic_command_option(self):
         def process_revision_directives(context, rev, generate_revisions):
@@ -608,7 +643,7 @@ def downgrade():
         assert (
             (
                 """
-def upgrade():
+def upgrade() -> None:
     # ### commands auto generated by Alembic - please adjust! ###
     op.create_table('test_table',
     sa.Column('id', sa.Integer(), nullable=False),
@@ -1194,9 +1229,9 @@ class DuplicateVersionLocationsTest(TestBase):
         )
 
         script = ScriptDirectory.from_config(self.cfg)
-        self.model1 = util.rev_id()
-        self.model2 = util.rev_id()
-        self.model3 = util.rev_id()
+        self.model1 = "ccc" + util.rev_id()
+        self.model2 = "bbb" + util.rev_id()
+        self.model3 = "aaa" + util.rev_id()
         for model, name in [
             (self.model1, "model1"),
             (self.model2, "model2"),
@@ -1236,9 +1271,9 @@ def downgrade():
         clear_staging_env()
 
     def test_env_emits_warning(self):
-        with assertions.expect_warnings(
+        msg = (
             "File %s loaded twice! ignoring. "
-            "Please ensure version_locations is unique"
+            "Please ensure version_locations is unique."
             % (
                 os.path.realpath(
                     os.path.join(
@@ -1248,7 +1283,8 @@ def downgrade():
                     )
                 )
             )
-        ):
+        )
+        with assertions.expect_warnings(msg, regex=False):
             script = ScriptDirectory.from_config(self.cfg)
             script.revision_map.heads
             eq_(
